package dev.shadowsoffire.apotheosis.adventure.loot;

import com.google.common.base.Predicates;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import java.util.function.Function;
import net.minecraft.util.ExtraCodecs;
import java.util.Collections;
import java.util.LinkedList;
import java.util.HashMap;
import net.minecraft.world.item.BowItem;
import net.minecraft.world.item.CrossbowItem;
import net.minecraft.world.item.TridentItem;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attributes;
import net.minecraftforge.common.ToolActions;
import net.minecraft.world.item.Item;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.block.AbstractSkullBlock;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.Items;
import java.util.Iterator;
import net.minecraftforge.registries.ForgeRegistries;
import dev.shadowsoffire.apotheosis.adventure.AdventureConfig;
import javax.annotation.Nullable;
import com.google.common.base.Preconditions;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.item.ItemStack;
import java.util.function.Predicate;
import java.util.Set;
import com.mojang.serialization.Codec;
import java.util.List;
import java.util.Map;

public final class LootCategory
{
    private static final Map<String, LootCategory> BY_ID_INTERNAL;
    private static final List<LootCategory> VALUES_INTERNAL;
    public static final Map<String, LootCategory> BY_ID;
    public static final List<LootCategory> VALUES;
    public static final Codec<LootCategory> CODEC;
    public static final Codec<Set<LootCategory>> SET_CODEC;
    public static final LootCategory BOW;
    public static final LootCategory CROSSBOW;
    public static final LootCategory PICKAXE;
    public static final LootCategory SHOVEL;
    public static final LootCategory HEAVY_WEAPON;
    public static final LootCategory HELMET;
    public static final LootCategory CHESTPLATE;
    public static final LootCategory LEGGINGS;
    public static final LootCategory BOOTS;
    public static final LootCategory SHIELD;
    public static final LootCategory TRIDENT;
    public static final LootCategory SWORD;
    public static final LootCategory NONE;
    private final String name;
    private final Predicate<ItemStack> validator;
    private final EquipmentSlot[] slots;
    
    private LootCategory(final String name, final Predicate<ItemStack> validator, final EquipmentSlot[] slots) {
        this.name = Preconditions.checkNotNull(name);
        this.validator = Preconditions.checkNotNull(validator);
        this.slots = Preconditions.checkNotNull(slots);
    }
    
    public String getDescId() {
        return "text.apotheosis.category." + this.name;
    }
    
    public String getDescIdPlural() {
        return this.getDescId() + ".plural";
    }
    
    public String getName() {
        return this.name;
    }
    
    public EquipmentSlot[] getSlots() {
        return this.slots;
    }
    
    public boolean isValid(final ItemStack stack) {
        return this.validator.test(stack);
    }
    
    public boolean isArmor() {
        return this == LootCategory.HELMET || this == LootCategory.CHESTPLATE || this == LootCategory.LEGGINGS || this == LootCategory.BOOTS;
    }
    
    public boolean isBreaker() {
        return this == LootCategory.PICKAXE || this == LootCategory.SHOVEL;
    }
    
    public boolean isRanged() {
        return this == LootCategory.BOW || this == LootCategory.CROSSBOW || this == LootCategory.TRIDENT;
    }
    
    public boolean isDefensive() {
        return this.isArmor() || this == LootCategory.SHIELD;
    }
    
    public boolean isLightWeapon() {
        return this == LootCategory.SWORD || this == LootCategory.TRIDENT;
    }
    
    public boolean isWeapon() {
        return this == LootCategory.SWORD || this == LootCategory.HEAVY_WEAPON || this == LootCategory.TRIDENT;
    }
    
    public boolean isWeaponOrShield() {
        return this.isLightWeapon() || this == LootCategory.SHIELD;
    }
    
    public boolean isNone() {
        return this == LootCategory.NONE;
    }
    
    @Override
    public String toString() {
        return String.format("LootCategory[%s]", this.name);
    }
    
    @Override
    public int hashCode() {
        return this.name.hashCode();
    }
    
    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof final LootCategory cat) {
            if (cat.name.equals(this.name)) {
                return true;
            }
        }
        return false;
    }
    
    public static final LootCategory register(@Nullable final LootCategory orderRef, final String name, final Predicate<ItemStack> validator, final EquipmentSlot[] slots) {
        final LootCategory cat = new LootCategory(name, validator, slots);
        if (LootCategory.BY_ID_INTERNAL.containsKey(name)) {
            throw new IllegalArgumentException("Cannot register a loot category with a duplicate name.");
        }
        LootCategory.BY_ID_INTERNAL.put(name, cat);
        int idx = LootCategory.VALUES_INTERNAL.size();
        if (orderRef != null) {
            idx = LootCategory.VALUES_INTERNAL.indexOf(orderRef);
        }
        LootCategory.VALUES_INTERNAL.add(idx, cat);
        return cat;
    }
    
    @Nullable
    public static LootCategory byId(final String name) {
        return LootCategory.BY_ID.get(name);
    }
    
    public static LootCategory forItem(final ItemStack item) {
        if (item.m_41619_()) {
            return LootCategory.NONE;
        }
        final LootCategory override = AdventureConfig.TYPE_OVERRIDES.get(ForgeRegistries.ITEMS.getKey((Object)item.m_41720_()));
        if (override != null) {
            return override;
        }
        for (final LootCategory c : LootCategory.VALUES) {
            if (c.isValid(item)) {
                return c;
            }
        }
        return LootCategory.NONE;
    }
    
    private static EquipmentSlot[] arr(final EquipmentSlot... s) {
        return s;
    }
    
    private static Predicate<ItemStack> armorSlot(final EquipmentSlot slot) {
        return stack -> {
            if (!stack.m_150930_(Items.f_42047_)) {
                final Item patt8272$temp = stack.m_41720_();
                if (patt8272$temp instanceof final BlockItem bi) {
                    if (bi.m_40614_() instanceof AbstractSkullBlock) {
                        return 0 != 0;
                    }
                }
                return LivingEntity.m_147233_(stack) == slot;
            }
            return false;
        };
    }
    
    static final LootCategory register(final String name, final Predicate<ItemStack> validator, final EquipmentSlot[] slots) {
        return register(null, name, validator, slots);
    }
    
    static {
        BY_ID_INTERNAL = new HashMap<String, LootCategory>();
        VALUES_INTERNAL = new LinkedList<LootCategory>();
        BY_ID = Collections.unmodifiableMap((Map<? extends String, ? extends LootCategory>)LootCategory.BY_ID_INTERNAL);
        VALUES = Collections.unmodifiableList((List<? extends LootCategory>)LootCategory.VALUES_INTERNAL);
        CODEC = ExtraCodecs.m_184405_((Function)LootCategory::getName, (Function)LootCategory::byId);
        SET_CODEC = PlaceboCodecs.setOf((Codec)LootCategory.CODEC);
        BOW = register("bow", s -> s.m_41720_() instanceof BowItem, arr(EquipmentSlot.MAINHAND, EquipmentSlot.OFFHAND));
        CROSSBOW = register("crossbow", s -> s.m_41720_() instanceof CrossbowItem, arr(EquipmentSlot.MAINHAND, EquipmentSlot.OFFHAND));
        PICKAXE = register("pickaxe", s -> s.canPerformAction(ToolActions.PICKAXE_DIG), arr(EquipmentSlot.MAINHAND));
        SHOVEL = register("shovel", s -> s.canPerformAction(ToolActions.SHOVEL_DIG), arr(EquipmentSlot.MAINHAND));
        HEAVY_WEAPON = register("heavy_weapon", new ShieldBreakerTest(), arr(EquipmentSlot.MAINHAND));
        HELMET = register("helmet", armorSlot(EquipmentSlot.HEAD), arr(EquipmentSlot.HEAD));
        CHESTPLATE = register("chestplate", armorSlot(EquipmentSlot.CHEST), arr(EquipmentSlot.CHEST));
        LEGGINGS = register("leggings", armorSlot(EquipmentSlot.LEGS), arr(EquipmentSlot.LEGS));
        BOOTS = register("boots", armorSlot(EquipmentSlot.FEET), arr(EquipmentSlot.FEET));
        SHIELD = register("shield", s -> s.canPerformAction(ToolActions.SHIELD_BLOCK), arr(EquipmentSlot.MAINHAND, EquipmentSlot.OFFHAND));
        TRIDENT = register("trident", s -> s.m_41720_() instanceof TridentItem, arr(EquipmentSlot.MAINHAND));
        SWORD = register("sword", s -> s.canPerformAction(ToolActions.SWORD_DIG) || s.m_41720_().getAttributeModifiers(EquipmentSlot.MAINHAND, s).get(Attributes.f_22281_).stream().anyMatch(m -> m.m_22218_() > 0.0), arr(EquipmentSlot.MAINHAND));
        NONE = register("none", (Predicate<ItemStack>)Predicates.alwaysFalse(), new EquipmentSlot[0]);
    }
}
